#include <stdio.h>
#include <string.h>

#include <firestorm.h>
#include <packet.h>
#include <plugin.h>
#include <generator.h>
#include <alert.h>
#include <signature.h>
#include <decode.h>
#include <preproc.h>
#include <netinet/in.h>

PLUGIN_STD_DEFS();

int http_dprint(struct layer *l, char *buf, int buflen)
{
	struct http_session *h=l->session;
	char *obuf=buf;

	/* 28 = 14+12+2 = extraneous bits of string */
	if ( !h ) return snprintf(buf, buflen, "No Information");
	if ( buflen < 28+h->uri_len+h->method_len )
		return snprintf(buf, buflen, "buffer too small");

	memcpy(buf, "http_method='", 13); buf+=13;
	memcpy(buf, h->method, h->method_len); buf+=h->method_len;
	memcpy(buf, "' http_uri='", 12); buf+=12;
	memcpy(buf, h->uri, h->uri_len); buf+=h->uri_len;
	memcpy(buf, "'", 2);

	return (buf-obuf);
}

void http_decode(struct packet *p) 
{
	struct pkt_iphdr *iph=p->layer[p->llen-2].h.ip;
	struct pkt_tcphdr *tcph=p->layer[p->llen-1].h.tcp;
	u_int32_t flags=p->layer[p->llen-1].flags;
	char *ptr=p->layer[p->llen].h.raw;
	char *end;
	size_t rlen=ntohs(iph->tot_len) - (iph->ihl<<2) - (tcph->doff<<2);
	size_t plen=(char *)p->end-ptr;
	size_t len;
	
	struct http_session h={NULL,0,ptr,0};
	int state;

	/* No special requirements */
	p->llen++;

	/* Just parse requests for now */
	if ( !(flags & FLAG_TCP_2SVR) ) goto done;

	/* work out min length to scan */
	len = ( plen<rlen ) ? plen : rlen;

	/* make sure we actually have a payload */
	if ( !len ) goto done;

	/* Really crap way of parsing out URIs, will
	 * fuck up as soon as we see a packet that
	 * doesn't start with an http request */
	for(state=0,end=ptr+len; ptr<end; ptr++) {
		if ( *ptr<32 || *ptr&0x80 ) {
			break;
		}
		if ( state==0 ) {
			if ( *ptr==' ' ) {
				state=1;
				continue;
			}
			h.method_len++;
		}else if ( state==1 ) {
			if ( *ptr!=' ' ) {
				h.uri=ptr;
				h.uri_len=1;
				state=2;
			}
		}else if ( state==2 ) {
			if ( *ptr==' ' ) break;
			h.uri_len++;
		}
	}

	if ( h.uri && h.uri_len && h.method_len ) {
		p->layer[p->llen].session=&h;
	}

done:
	return;
}

int _PLUGIN_INIT (struct plugin_in *in, struct plugin_out *out)
{
	static struct proto http_p=init_sproto("http",
		http_decode, http_dprint,
		NULL, NULL,
		PSF_REQUEST);
	static struct proto_req http_r[]={
		proto_request("tcp", __constant_htons(80)),
		proto_request("tcp", __constant_htons(3128)),
		proto_request("tcp", __constant_htons(8080)),
		null_request()
	};
	proc_decode_add decode_add;
	int ok=0;
	
	plugin_check(in, out);
	
	PLUGIN_ID("decode.http", "Hyper-text Transfer Protocol");
	PLUGIN_VERSION(0, 1);
	PLUGIN_AUTHOR("Gianni Tedesco", "gianni@scaramanga.co.uk");
	PLUGIN_LICENSE("GPL");
	
	if ( !(decode_add=in->import("decode.add")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	ok+=decode_add(&http_p, http_r);
	
	return (ok) ? PLUGIN_ERR_OK : PLUGIN_ERR_FAIL;
}

int _PLUGIN_UNLOAD (int code) {
	return PLUGIN_ERR_OK;
}
